SystemOrganization addCategory: #DerivedObject!

ProtoObject subclass: #DerivedObject
	instanceVariableNames: 'theNew origin'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DerivedObject'!

!DerivedObject commentStamp: 'lp 4/1/2020 14:50' prior: 0!
A DerivedObject is two (or more) objects in one.

Derived objects implement ad-hoc, non-destructive, primitive multiclassing. Make derived objects with #derive:, which takes a class. You can get the original object back with #:origin. You can test if the object is derived from another object with #isDerived.

**** Non-destructive

Deriving an object does not modify the state of the original object. You end up with an object with private, freshly-derived state, and shared state. You can use the same original object to derive many objects. You can even derive an object of the same class; think of it like a change set.!

!DerivedObject class methodsFor: 'initialize-release' stamp: 'lp 4/1/2020 14:29'!
derive: aClass from: anObject

	^ DerivedObject new origin: anObject; theNew: aClass new! !

!DerivedObject methodsFor: 'private' stamp: 'lp 4/1/2020 14:23'!
doesNotUnderstand: aMessage

	"I trampoline the message to my receiver's parts."
	| returning |
	(theNew respondsTo: aMessage selector)
		ifTrue: [ returning := theNew perform: aMessage selector withArguments: aMessage arguments ]
		ifFalse: [ returning := origin perform: aMessage selector withArguments: aMessage arguments ].
	returning = theNew ifTrue: [ ^ self ].
	returning = origin ifTrue: [ ^ self ].
	^ returning! !

!DerivedObject methodsFor: 'accessing' stamp: 'lp 4/1/2020 14:02'!
isDerived

	^ true! !

!DerivedObject methodsFor: 'accessing' stamp: 'lp 4/1/2020 14:24'!
origin

	^ origin! !

!DerivedObject methodsFor: 'private' stamp: 'lp 4/1/2020 14:27'!
origin: anObject

	origin = nil ifFalse: [ ^ 'can only call #origin: once' error].
	origin := anObject! !

!DerivedObject methodsFor: 'private' stamp: 'lp 4/1/2020 14:25'!
respondsTo: aSymbol

	"I trampoline the message to my receiver's parts."
	(theNew respondsTo: aSymbol)
		ifTrue: [ ^ true ]
		ifFalse: [ ^ origin respondsTo: aSymbol ]! !

!DerivedObject methodsFor: 'private' stamp: 'lp 4/1/2020 14:28'!
theNew: anObject

	theNew = nil ifFalse: [ ^ 'can only call #theNew: once' error ].
	theNew := anObject! !

!ProtoObject methodsFor: '*DerivedObject' stamp: 'lp 4/1/2020 14:29'!
derive: aClass

	^ DerivedObject derive: aClass from: self! !

!ProtoObject methodsFor: '*DerivedObject' stamp: 'lp 4/1/2020 14:03'!
isDerived

	^ false! !
